#!/bin/bash
# Update-post: functions for the script
# by join, 2009

# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What The Fuck You Want
# To Public License, Version 2, as published by Sam Hocevar. See
# http://sam.zoy.org/wtfpl/COPYING for more details.

# Changelog :
#
# 10/02/2009	Changed the method for retrieving absolute paths from
#	the readlink method to a more portable method. The previous bug
#	was also not very efficient, so now it is fixed.
# 07/02/2009	Corrected a bug about recursive sorting (where the
#	Sorted dir is placed into the Global dir)
# 02/02/2009	Creation.

BASE_DIR="$(pwd)"
SORT_DIR="/home/join/EnCours/Sorted"
GLOBAL_DIR="/home/join/EnCours"


# What is the date today ?
date_mth=$(date +%Y_%B)
date_day=$(date +%d_%A)

dir_cpt="$date_mth/$date_day"

function real_path() {
	local filename="$*"
	[ -e "$filename" ] || exit 1 
	[ -d "$filename" ] || filename="$(dirname "$filename")"
	(cd "$filename"; pwd)
}

function check_directories() {
	# Check if the sort directory exists
	if [ ! -d "$SORT_DIR" ]; then
		echo "Creating the Sorted directory..."
		mkdir "$SORT_DIR"
	fi

	# Check if the global directory exists
	if [ ! -d "$GLOBAL_DIR" ]; then
		echo "Creating the Global directory..."
		mkdir "$GLOBAL_DIR"
	fi
}

function snapshot() {
	(cd "$GLOBAL_DIR"; find .)
}


function check_folder_snapshot() {
	# Check if a folder snapshot already exists or not
	if [ ! -f "$SORT_DIR/.snapshot" ]; then
		echo "Creating a new snapshot..."
		snapshot > "$SORT_DIR/.snapshot"
	fi

	# Check if a date is given
	if [ ! -f "$SORT_DIR/.snapshot_date" ]; then
		echo "Writing out snapshot date..."
		echo "$dir_cpt" > "$SORT_DIR/.snapshot_date"
	fi

}

function we_need_update() {
	# Determine whether we need to have a new directory
	local date_snap=$(cat "$SORT_DIR/.snapshot_date")
	[ "$dir_cpt" == "$date_snap" ] && return 1
	return 0
}

function compare_snapshot() {
	# Take the current snapshot
	snapshot > "$SORT_DIR/.snapshot2"

	# Let's compare them
	diff "$SORT_DIR/.snapshot" "$SORT_DIR/.snapshot2" > "$SORT_DIR/.snapshot_diff"

	# If we need to set a new date then let's update NOW !
	if we_need_update; then
		# Create a new folder representing today !
		mkdir -p "$SORT_DIR/$dir_cpt"
		# Change snapshot
		rm "$SORT_DIR/.snapshot"
		rm "$SORT_DIR/.snapshot_date"
		check_folder_snapshot
	else
		# Let's take a new snapshot
		snapshot > "$SORT_DIR/.snapshot"
	fi

	# We have to compare the already existing snapshot
	dest_dir=$(cat "$SORT_DIR/.snapshot_date")
	complete_dest_dir="$SORT_DIR/$dest_dir"

	# Check if the complete dest dir is created
	if [ ! -d "$complete_dest_dir" ]; then
		echo "Creating a new directory in the sorted structure : $dest_dir..."
		mkdir -p "$complete_dest_dir"
	fi

	IFS=$'\n'

	# Let's destroy what we have to destroy
	# We have to reverse the order of the files because they must be destroyed from
	# the deepest element to the nearest.
	for file in $(cat "$SORT_DIR/.snapshot_diff" | grep [\<] | cut -c3- | sort -r); do
		# And remember : file is relative to $GLOBAL_DIR, and to our dest dir !
		real_file="$complete_dest_dir/$file"
		# Be sure not to operate recursively in our Sorted directory
		[[ "$(real_path $real_file)" =~ ^"$SORT_DIR" ]] && continue
		echo "Removing $file..."
		# Then we are ready to destroy !
		if [ -L "$real_file" ]; then
			# Destroy a link : rm
			rm "$real_file"
		elif [ -d "$real_file" ]; then
			# Destroy an empty folder (if not, then there's a problem)
			rmdir "$real_file"
		fi
		# And that's all folks !
	done
	
	# Let's link what we have to link
	for file in $(cat "$SORT_DIR/.snapshot_diff" | grep [\>] | cut -c3-); do
		# And remember : file is relative to $GLOBAL_DIR !
		real_file="$GLOBAL_DIR/$file"
		# Be sure not to operate recursively in our Sorted directory
		[[ "$(real_path $real_file)" =~ ^"$SORT_DIR" ]] && continue
		echo "Adding $file..."
		# Ready to link, captain !
		if [ -d "$real_file" ]; then
			# Create a directory
			mkdir -p "$complete_dest_dir/$file"
		else
			# Verify there's a directory leading to that file,
			# otherwise create it !
			if [ ! -d "$(dirname "$complete_dest_dir/$file")" ]; then
				echo "Creating missing directories..."
				mkdir -p "$(dirname "$complete_dest_dir/$file")"
			fi
			# Create an absolute symlink
			ln -s "$real_file" "$complete_dest_dir/$file"
		fi
		# Finished !
	done
	unset IFS
}

function check_bad_symlinks() {
	echo "Checking for bad symlinks..."
	IFS=$'\n'
	for file in $(find "$SORT_DIR/" -type l); do
		if [ ! -e "$(readlink "$file")" ]; then
			rm "$file"
			echo "Bad symlink $(basename "$file") deleted."
		fi
	done
	unset IFS
}

function check_empty_folders() {
	echo "Checking for empty folders..."
	IFS=$'\n'
	for file in $(find "$SORT_DIR/" -type d); do
		if [ $(ls "$file" | wc -l) -eq 0 ]; then
			rmdir "$file"
			echo "Empty folder $(basename $file) deleted."
		fi
	done
	unset IFS
}
